@@ -1,15 +1,15 @@
 #ifndef __PFE_DEF_COMP_H
-#define __PFE_DEF_COMP_H "%full_filespec: def-comp.h~bln_mpt1!5.22:incl:bln_12xx!1 %"
-
+#define __PFE_DEF_COMP_H
 /* 
  * -- internal structure definitions 
  *
- *  Copyright (C) Tektronix, Inc. 1998 - 2001. All rights reserved.
+ *  Copyright (C) Tektronix, Inc. 1998 - 2003.
+ *  Copyright (C) 2005 - 2008 Guido U. Draheim <guidod@gmx.de>
  *
  *  @see     GNU LGPL
- *  @author  Tektronix CTE             @(#) %derived_by: guidod %
- *  @version %version: bln_mpt1!5.22 %
- *    (%date_modified: Tue Oct 09 13:08:10 2001 %)
+ *  @author  Guido U. Draheim            (modified by $Author: guidod $)
+ *  @version $Revision: 1.7 $
+ *     (modified $Date: 2008/05/11 21:10:21 $)
  *
  *  @description
  * 	lists of declarations reflecting internal structures
@@ -17,7 +17,8 @@
  */
 /*@{*/
 
-#include <pfe/incl-sub.h>
+#include <pfe/pfe-sub.h>
+#include <pfe/def-types.h>
 
 #ifndef P4STRING
 # if defined PFE_OLD_STRINGIZE
@@ -34,6 +35,13 @@
 #define P4STRING(A) P4STR(A)
 #endif
 
+#ifdef PFE_SUFFIX
+#define P4SUFFIX_(A,B,C) P4CAT3(A,B,C)
+#define P4SUFFIX(A,B) P4SUFFIX_(A,B,PFE_SUFFIX)
+#else
+#define P4SUFFIX(A,B) P4CAT(A,B)
+#endif
+
 #define P4CODE(X) P4CAT(X,_)
 
 /* only works in 0.31.x style !! */
@@ -91,8 +99,8 @@ typedef void (*p4code_RT) (P4_ARG_TAKE_BODY_T);
 /* p4_destroyer_RT somewhere else :eek: */
 /* btw, watch out not to jump on p4_name_from and p4_obsoleted_RT */
 # define PFE_IS_DESTROYER(__nfa) \
-   ( (P4xONxDESTROY   &&  *_FFA(__nfa) & P4xONxDESTROY) \
-    || (*_FFA(__nfa) & P4xIMMEDIATE \
+   ( (P4xONxDESTROY   &&  P4_NFA_FLAGS(__nfa) & P4xONxDESTROY) \
+    || (P4_NFA_FLAGS(__nfa) & P4xIMMEDIATE \
         && P4_XT_VALUE(P4_LINK_FROM(p4_name_to_link(__nfa))) \
 	   == FX_GET_RT(p4_destroyer)))
 
@@ -111,8 +119,8 @@ typedef struct p4_Decomp p4_Decomp; /* informations for the decompiler */
 typedef struct p4_Semant p4_Semant; /* pointer set for state smart words */
 typedef struct p4_Seman2 p4_Seman2; /* dito for even smarter words like TO */
 
-#define P4_CODE_SEE(func) p4xt* func (p4xt* ip, char* p, p4_Semant* s)
-#define P4_CODE_RUN(func) p4xt* func (char* p, p4xt xt, p4char* nfa)
+#define P4_CODE_SEE(func) p4xcode* func (p4xcode* ip, char* p, p4_Semant* s)
+#define P4_CODE_RUN(func) p4xcode* func (char* p, p4xt xt, p4char* nfa)
 
 /* encodings for what information follows the compiled word inline */
 #define  P4_SKIPS_NOTHING            ((P4_CODE_SEE((*)))(0))
@@ -147,7 +155,7 @@ struct p4_Semant		/* for words with different compilation */
 {				/* and execution semantics: */
     long magic;			/* mark begin of structure */
     p4_Decomp decomp;		/* decompiler aid */
-    char const *name;		/* compiled by */
+    p4_namebuf_t const *name;	/* compiled by */
     p4code comp;		/* compilation/interpretation semantics */
     p4code exec[1];		/* execution semantics */
 };
@@ -156,7 +164,7 @@ struct p4_Seman2		/* for words with different compilation */
 {				/* and two different execution semantics: */
     long magic;			/* mark begin of structure */
     p4_Decomp decomp;		/* decompiler aid */
-    char const *name;		/* compiled by */
+    p4_namebuf_t const *name;	/* compiled by */
     p4code comp;		/* compilation/interpretation semantics */
     p4code_XE exec[2];		/* two different execution semantics */
 };				/* for cases like TO (value/local variable) */
@@ -176,8 +184,8 @@ struct p4_Runtime2              /* describes characteristics of CFA code */
     } run;                      /* we did not make an extra typedef for it */
 };
 
-#define P4SEMANTICS(X) P4CAT(X,_Semant)
-#define P4RUNTIME_(X) P4CAT(X,_Runtime)
+#define P4SEMANTICS(X) P4SUFFIX(X,_Semant)
+#define P4RUNTIME_(X) P4SUFFIX(X,_Runtime)
 
 #ifndef HOST_WIN32
 #define P4COMPILES(C,E,S,STYLE)			\
@@ -200,26 +208,26 @@ p4_Seman2 P4SEMANTICS(C) =			\
   { P4CODE (E1), P4CODE (E2) }			\
 }
 
-#define P4RUNTIMES1(C,E1,FLAGS)                 \
+#define P4RUNTIMES1_(C,E1,FLAGS,SEE)            \
 p4_Runtime2 P4RUNTIME_(C) =                     \
-{ "@",                                          \
+{ (const p4char*) "@",                          \
   P4_RUNTIME_MAGIC, FLAGS, 0,                   \
   P4CODE(C), { P4CODE(E1), NULL },              \
-  { NULL, NULL, NULL }                          \
+  { SEE, NULL, NULL }                           \
 }
 
-#define P4RUNTIMES2(C,E1,E2,FLAGS)              \
+#define P4RUNTIMES2_(C,E1,E2,FLAGS,SEE)         \
 p4_Runtime2 P4RUNTIME_(C) =                     \
-{ "@",                                          \
+{ (const p4char*) "@",                          \
   P4_RUNTIME_MAGIC, FLAGS, 0,                   \
   P4CODE(C), { P4CODE(E1), P4CODE(E2) },        \
-  { NULL, NULL, NULL },                         \
+  { SEE, NULL, NULL },                          \
 }
 
 #else /* HOST_WIN32 */
 
 #define P4COMPILES(C,E,S,STYLE)			\
-p4_Semant* P4SEMANTICS(C) (void)			\
+p4_Semant* P4SEMANTICS(C) (void)		\
 { static p4_Semant semant =			\
  {						\
   P4_SEMANT_MAGIC,				\
@@ -242,28 +250,31 @@ p4_Seman2* P4SEMANTICS(C) (void)		\
  }; return &semant;                             \
 }
 
-#define P4RUNTIMES1(C,E,FLAGS)                  \
+#define P4RUNTIMES1_(C,E,FLAGS,SEE)             \
 p4_Runtime2* P4RUNTIME_(C) (void)               \
 { static p4_Runtime2 runtime =                  \
- { "@",                                         \
+ { (const p4char*) "@",                         \
   P4_RUNTIME_MAGIC, FLAGS, 0,                   \
   P4CODE(C), { P4CODE(E), NULL },               \
-  { NULL, NULL, NULL }                          \
+  { SEE, NULL, NULL }                           \
  }; return &runtime;                            \
 }
 
-#define P4RUNTIMES2(C,E1,E2,FLAGS)              \
+#define P4RUNTIMES2_(C,E1,E2,FLAGS,SEE)         \
 p4_Runtime2* P4RUNTIME_(C) (void)               \
 { static p4_Runtime2 runtime =                  \
- { "@",                                         \
+ { (const p4char*) "@",                         \
   P4_RUNTIME_MAGIC, FLAGS, 0,                   \
   P4CODE(C), { P4CODE(E1), P4CODE(E2) },        \
-  { NULL, NULL, NULL }                          \
+  { SEE, NULL, NULL }                           \
  }; return &runtime;                            \
 }
 
 #endif /* HOST_WIN32 */
 
+#define P4RUNTIMES1(C,E,FLAGS) P4RUNTIMES1_(C,E,FLAGS,0)
+#define P4RUNTIMES2(C,E1,E2,FLAGS) P4RUNTIMES1_(C,E1,E2,FLAGS,0)
+
 /* compile execution semantics from within C-code: */
 #ifndef HOST_WIN32
 #define FX_DEF_COMPILES(X) p4_Semant  P4SEMANTICS(X)
@@ -306,9 +317,9 @@ p4_Runtime2* P4RUNTIME_(C) (void)               \
 #define FX_COMPILE_COMMA_RT(X) FX_ZCOMMA(X)
 #else
 #define FX_COMPILE_COMMA_XE(X) \
-             p4_DP_CODE = p4_compile_xcode_CODE(p4_DP_CODE,X)
+             p4_DP = (p4char*) p4_compile_xcode_CODE((p4xcode*)p4_DP,X)
 #define FX_COMPILE_COMMA_RT(X,B) \
-             p4_DP_CODE = p4_compile_xcode_BODY(p4_DP_CODE,X,B)
+             p4_DP = (p4char*) p4_compile_xcode_BODY((p4xcode*)p4_DP,X,B)
 #endif
 
 #define FX_COMPILE(X)  do { extern     FX_DEF_COMPILES(X);  \
@@ -326,7 +337,7 @@ p4_Runtime2* P4RUNTIME_(C) (void)               \
 #if   ! defined PFE_CALL_THREADING
 #define P4_XT_VALUE(__xt) (*(__xt))
 #define FX_GET_RT(__rt)   __rt##_RT_ 
-#define extern_FX_DEF_RUNTIME1(X) /* nothing */
+#define extern_FX_DEF_RUNTIME1(X) extern FX_DEF_RUNTIME1(p4_colon) /* nothing -> duplicate */
 #define FX_RUNTIME1_RT(X) FX_XCOMMA(X##_RT_) /* a simply comma */
 #define P4RUNTIME1_RT(X) /* nothing */
 #define P4RUNTIMES1_RT(X,F) /* nothing */
@@ -340,9 +351,13 @@ p4_Runtime2* P4RUNTIME_(C) (void)               \
                                                      P4RUNTIMES1(X, X##_RT, F)
 #endif
 
-#define FX_IMMEDIATE          *NFA2FF(p4_LAST) |= P4xIMMEDIATE
-#define FX_SMUDGED            *NFA2FF(p4_LAST) |= P4xSMUDGED
-#define FX_RUNTIME_BODY       *NFA2FF(p4_LAST) |= P4xISxRUNTIME
+#define FX_IMMEDIATE          P4_NFA_FLAGS(p4_LAST) |= P4xIMMEDIATE
+#define FX_SMUDGED            P4_NFA_FLAGS(p4_LAST) |= P4xSMUDGED
+#define FX_RUNTIME_BODY       P4_NFA_FLAGS(p4_LAST) |= P4xISxRUNTIME
+
+#define P4_NFA_xIMMEDIATE(X)    (P4_NFA_FLAGS(X) & P4xIMMEDIATE)
+#define P4_NFA_xSMUDGED(X)      (P4_NFA_FLAGS(X) & P4xSMUDGED)
+#define P4_NFA_xISxRUNTIME(X)   (P4_NFA_FLAGS(X) & P4xISxRUNTIME)
 
 #if   ! defined PFE_CALL_THREADING
 #define FX_POP_BODY_ADDR           (P4_WP_PFA)
@@ -350,15 +365,15 @@ p4_Runtime2* P4RUNTIME_(C) (void)               \
 #define FX_POP_BODY_ADDR_p4_BODY   register p4cell* p4_BODY = FX_POP_BODY_ADDR
 #define FX_POP_BODY_ADDR_UNUSED
 #elif ! defined PFE_SBR_CALL_THREADING
-#define FX_POP_BODY_ADDR           ((p4cell*)(*IP++))
+#define FX_POP_BODY_ADDR           ((p4cell*)(*p4IP++))
 #define FX_POP_BODY_ADDR_(VARNAME) register p4cell* VARNAME = FX_POP_BODY_ADDR
 #define FX_POP_BODY_ADDR_p4_BODY   register p4cell* p4_BODY = FX_POP_BODY_ADDR
-#define FX_POP_BODY_ADDR_UNUSED    (IP++)
+#define FX_POP_BODY_ADDR_UNUSED    (p4IP++)
 #elif ! defined PFE_SBR_CALL_ARG_THREADING
-#define FX_POP_BODY_ADDR           ((p4cell*)(*(*PFE_SBR_RP)++))
+#define FX_POP_BODY_ADDR           ((p4cell*)(*p4IP++))
 #define FX_POP_BODY_ADDR_(VARNAME) register p4cell* VARNAME = FX_POP_BODY_ADDR
 #define FX_POP_BODY_ADDR_p4_BODY   register p4cell* p4_BODY = FX_POP_BODY_ADDR
-#define FX_POP_BODY_ADDR_UNUSED    (++(*PFE_SBR_RP))
+#define FX_POP_BODY_ADDR_UNUSED    (++p4IP)
 #else
 #define FX_POP_BODY_ADDR           (p4_BODY)
 #define FX_POP_BODY_ADDR_(VARNAME) register p4cell* VARNAME = FX_POP_BODY_ADDR
@@ -370,7 +385,7 @@ p4_Runtime2* P4RUNTIME_(C) (void)               \
 #define FX_COMPILE_COMMA(xt) FX_XCOMMA(xt)
 #else
 #define FX_COMPILE_COMMA(xt) \
-        p4_DP_CODE = p4_compile_comma(p4_DP_CODE, xt)
+        p4_DP = (p4char*) p4_compile_comma((p4xcode*)p4_DP, xt)
 #endif
 
 #ifndef PFE_CALL_THREADING
@@ -396,11 +411,40 @@ p4_Runtime2* P4RUNTIME_(C) (void)               \
 #define P4RUNTIME1(C,E1)    P4RUNTIMES1(C,E1,0)
 #define P4RUNTIME2(C,E1,E2) P4RUNTIMES2(C,E1,E2,0)
 
+/* ------------------------------------------------------------------ */
+/* debug-ext: we walk registered wordsets (in atexit_wl) looking for
+ * items that can be used for decompiling. No need to register each
+ * item in the forth dictionary anymore. Here we define a walker struct
+ * that can be referenced in decompiler-routines in the wordsets around
+ */
+
+typedef struct p4_Decompile p4_Decompile;
+struct p4_Decompile
+{
+    p4char* next;            /* initially set to PFE.atexit_wl->thread[0] */
+    char const* wordset;     /* current wordset reference, the name info */ 
+    int left;                /* how many left to check: */
+    struct {                 /* compatible with p4Words !! */
+	struct{
+	    const char type;    /* loader-type */
+	    const char lencode; /* forth-name compatibility */
+	    const char name[1]; /* zero-terminated... */
+	} *loader;
+	union {                 /* the value part from the wordset item: */
+	    const p4cell cell;             /* as type p4cell */
+	    void * ptr;                    /* generic pointer */
+	    p4_Runtime2 const * runtime;   /* runtime reference */
+	    p4_Seman2   const * semant;    /* semant reference */
+	    char* name;                    /* name, zero-terminated */
+	} value;
+    } *word;
+};
+
 /* ------------------------------------------------------------------ */
 
 p4_Semant* p4_to_semant(p4xt xt);
 char p4_category (p4code p);
-void p4_decompile (char *nfa, p4xt xt);
+void p4_decompile (p4_char_t* nfa, p4xt xt);
 
 FCode (p4_noop);
 FCode (p4_ahead);
@@ -416,7 +460,7 @@ FCode (p4_unsmudge);
 
 /* Local variables: */
 
-int p4_find_local (char *nm, int l);
+int p4_find_local (const p4_char_t *nm, int l);
 int p4_word_compile_local (void);
 
 /* Runtimes to identify words by them: */
@@ -439,27 +483,27 @@ FCode_RT (p4_builds_RT);	extern FX_DEF_RUNTIME1(p4_builds);
 FCode_RT (p4_variable_RT);	extern FX_DEF_RUNTIME1(p4_variable);
 FCode_RT (p4_constant_RT);	extern FX_DEF_RUNTIME1(p4_constant);
 FCode_RT (p4_value_RT);		extern FX_DEF_RUNTIME1(p4_value);
-FCode_RT (p4_two_constant_RT);
-FCode_RT (p4_f_constant_RT);
-FCode_RT (p4_f_variable_RT);
+FCode_RT (p4_two_constant_RT);  extern FX_DEF_RUNTIME1(p4_two_constant);
+FCode_RT (p4_f_constant_RT);    extern FX_DEF_RUNTIME1(p4_f_constant);
+FCode_RT (p4_f_variable_RT);    extern FX_DEF_RUNTIME1(p4_f_variable);
 
 FCode_RT (p4_colon_RT);		extern FX_DEF_RUNTIME1(p4_colon);
 FCode_RT (p4_debug_colon_RT);	extern FX_DEF_RUNTIME1(p4_debug_colon);
-FCode_RT (p4_dictvar_RT);		extern_FX_DEF_RUNTIME1(p4_dictvar);
-FCode_RT (p4_dictget_RT);		extern_FX_DEF_RUNTIME1(p4_dictget);
-FCode_RT (p4_dictset_RT);		extern_FX_DEF_RUNTIME1(p4_dictset);
-FCode_RT (p4_destroyer_RT);        extern_FX_DEF_RUNTIME1(p4_destroyer);
+FCode_RT (p4_dictvar_RT);	extern_FX_DEF_RUNTIME1(p4_dictvar);
+FCode_RT (p4_dictget_RT);	extern_FX_DEF_RUNTIME1(p4_dictget);
+FCode_RT (p4_destroyer_RT);     extern_FX_DEF_RUNTIME1(p4_destroyer);
 
 FCode    (p4_only_RT);		extern_FX_DEF_RUNTIME1(p4_only);
 FCode    (p4_forth_RT);		extern_FX_DEF_RUNTIME1(p4_forth);
 FCode_RT (p4_vocabulary_RT);	extern FX_DEF_RUNTIME1(p4_vocabulary);
 FCode_RT (p4_does_RT);		extern FX_DEF_RUNTIME1(p4_does);
 FCode_RT (p4_debug_does_RT);	extern FX_DEF_RUNTIME1(p4_debug_does);
-FCode_RT (p4_marker_RT);		extern FX_DEF_RUNTIME1(p4_marker);
+FCode_RT (p4_marker_RT);	extern FX_DEF_RUNTIME1(p4_marker);
 FCode_RT (p4_defer_RT);		extern FX_DEF_RUNTIME1(p4_defer);
-FCode_RT (p4_offset_RT);		extern FX_DEF_RUNTIME1(p4_offset);
-FCode_RT (p4_synonym_RT);		extern FX_DEF_RUNTIME1(p4_synonym);
+FCode_RT (p4_offset_RT);	extern FX_DEF_RUNTIME1(p4_offset);
+FCode_RT (p4_synonym_RT);	extern FX_DEF_RUNTIME1(p4_synonym);
 FCode_RT (p4_obsoleted_RT);	extern FX_DEF_RUNTIME1(p4_obsoleted);
+FCode_RT (p4_deprecated_RT);    extern FX_DEF_RUNTIME1(p4_deprecated);
 
 #ifdef _DEBUG_BORLAND_ANSI_COMPILERS_ /* USER-CONFIG */
 /* adding SEMANT forward declarations... */
