@@ -1,15 +1,15 @@
 #ifndef __PFE_DEF_TYPES_H
-#define __PFE_DEF_TYPES_H "%full_filespec: def-types.h~bln_mpt1!5.47:incl:bln_12xx!1 %"
-
+#define __PFE_DEF_TYPES_H
 /** 
  * -- pfe's data structures
  *
- *  Copyright (C) Tektronix, Inc. 1998 - 2001. All rights reserved.
+ *  Copyright (C) Tektronix, Inc. 1998 - 2003.
+ *  Copyright (C) 2005 - 2008 Guido U. Draheim <guidod@gmx.de>
  *
  *  @see     GNU LGPL
- *  @author  Tektronix CTE              @(#) %derived_by: guidod %
- *  @version %version: bln_mpt1!5.47 %
- *    (%date_modified: Wed Jul 17 12:46:02 2002 %)
+ *  @author  Guido U. Draheim            (modified by $Author: guidod $)
+ *  @version $Revision: 1.8 $
+ *     (modified $Date: 2008/05/01 21:49:01 $)
  *
  *  @description
  *     p4th's forth data structures definitions, types and sys.names
@@ -17,15 +17,18 @@
  */
 /*@{*/
 
-#include <pfe/incl-ext.h>
+#include <pfe/def-config.h>
+#include <pfe/def-regs.h>
 
-#include <setjmp.h>
+/* typedef FILE */
 #include <stdio.h>
+#include <pfe/os-setjmp.h>
 
 #if defined HAVE_SYS_TYPES_H || defined PFE_HAVE_SYS_TYPES_H
 # include <sys/types.h>		/* size_t, time_t and friends */
 #endif
 
+#include <pfe/def-macro.h>
 #include <pfe/def-paths.h>
 
 #include <pfe/lined.h>
@@ -110,6 +113,11 @@
   
 #endif
 
+typedef p4char p4_namechar_t;      /* word list name char */
+typedef p4char p4_namebuf_t;       /* start of counted string with namechars */
+typedef p4char p4_charbuf_t;       /* start of counted string with i/o chars */
+/*   */        /* char */          /* i/o char of Standard C/C++ (compiler) */
+
 typedef p4cell  (*p4cell_p4code) (void); /* very useful sometimes */
 typedef p4ucell (*p4ucell_p4code) (void); /* very useful sometimes */
 
@@ -122,14 +130,16 @@ typedef struct p4_Except p4_Except; /* an exception frame */
 
 typedef p4char* pfe_lfa_t;
 typedef p4code  pfe_cfa_t;
-typedef int (*p4_decompile_func_t)(char* nfa, p4xt xt);
+typedef int (*p4_decompile_func_t)(p4_namebuf_t* nfa, p4xt xt);
+
+typedef struct { p4_byte_t buffer[P4_POCKET_SIZE]; } p4_pocket_t;
 
 struct p4_Wordl			/* a word list */
 {				/* a word list identifier is a (Wordl *) */
-    p4char* thread[P4_THREADS]; /* field of pointers to first NFA of chain */
+    p4_namebuf_t* thread[P4_THREADS]; /* field of ptrs to first NFA of chain */
     p4_Wordl *prev;		/* word lists are chained */
     p4_Wordl *also;		/* search also this dictionary, kinda FIG*/
-    p4char* nfa;                /* corresponding vocabulary NFA */
+    p4_namebuf_t* nfa;          /* corresponding vocabulary NFA */
     p4cell  flag;		/* do to_upper, a copy of sys.world_flag */
 #define WORDL_NOCASE     (1<<0) /* FIND will match any case with any case */
 #define WORDL_NOHASH     (1<<1) /* WORDLIST not hashed into multiple buckets */
@@ -139,19 +149,23 @@ struct p4_Wordl			/* a word list */
     p4ucell id;                 /* used to speed up order search (p4_find) */
 };
 
+/* FIXME: p4_File.n should be p4_File.block.num or p4_File.file.line */
+/* and similar prefix all block/file items using a local struct wrapper */
+
 struct p4_File			/* describes a file */
 {
-    FILE *f;			/* associated ANSI-C file structure */
-    char mdstr[4];		/* mode string for fopen() */
-    char mode;			/* mode code for open_file() */
-    signed char last_op;	/* -1 write, 0 none, 1 read */
-    p4word len;			/* if stream: length of input line */
-    p4ucell size;		/* if block file: size of file in blocks */
-    long pos;			/* a saved position, e.g. beginning of line */
-    p4ucell n;			/* block in buffer or source line */
-    p4cell updated;		/* if block file: block updated? */
-    char name[PATH_LENGTH];	/* file name */
-    char buffer[P4_BLK_SIZE];	/* buffer for block or input line */
+    FILE *f;			   /* associated ANSI-C file structure */
+    char mdstr[4];		   /* mode string for fopen() */
+    char mode;			   /* mode code for open_file() */
+    signed char last_op;	   /* -1 write, 0 none, 1 read */
+    p4word len;			   /* if stream: length of input line */
+    p4ucelll size;		   /* if block file: size of file in blocks */
+    p4ucelll n;			   /* block in buffer or source line */
+    p4cell updated;		   /* if block file: block updated? */
+    union { _p4_off_t pos;	   /* saved position, e.g. beginning of line */
+        char compat[8]; } line;
+    char name[PATH_LENGTH];	   /* file name */
+    p4_byte_t buffer[P4_BLK_SIZE]; /* buffer for block or input line */
 };
 
 struct p4_Input			/* an input source specification */
@@ -159,8 +173,9 @@ struct p4_Input			/* an input source specification */
     p4cell source_id;		/* SOURCE-ID */
     p4_File *block_file;	/* which file is active? */
     p4ucell blk;		/* currently loaded block */
-    long pos;			/* stream file: beginning of line in file */
-    char *tib;			/* points to TIB or to EVALUATED string */
+    union { _p4_off_t pos;	/* stream file: beginning of line in file */
+        char compat[8]; } line;
+    const p4_char_t *tib;	/* points to TIB or to EVALUATED string */
     p4ucell number_tib;		/* #TIB, length of string in TIB */
     p4ucell to_in;		/* input parsing position */
 };
@@ -175,12 +190,13 @@ struct p4_Iframe		/* a frame to save an input specification */
 struct p4_Except
 {
     p4cell magic;
-    p4xcode *ipp;
-    p4cell *spp;
-    p4cell *lpp;
-    double *fpp;
+    p4xcode** rpp;              /* P4_REGRP_T */
+    p4xcode *ipp;               /* P4_REGIP_T */
+    p4cell *spp;                /* P4_REGSP_T */
+    p4cell *lpp;                /* P4_REGLP_T */
+    double *fpp;                /* P4_REGFP_T */
     p4_Iframe *iframe;
-    jmp_buf jmp;
+    p4_jmp_buf jmp;
     p4_Except *prev;
 };
 
@@ -210,11 +226,11 @@ typedef struct p4_Dictionary    p4_Dictionary;
 
 struct p4_Dictionary
 {
-    p4char* last;  /* PFE.last -> PFE.dict.last */
-    p4char* link;  /* PFE.link -> PFE.dict.link */
-    p4char* here;  /* PFE.dp   -> PFE.dict.here */
-    p4char* base;  /* PFE.dict -> PFE.dict.base */
-    p4char* limit; /* PFE.dictlimit -> PFE.dict.limit */
+    p4_namebuf_t* last; /* PFE.last -> PFE.dict.last */
+    p4_namebuf_t* link;  /* PFE.link -> PFE.dict.link */
+    p4_byte_t* here;     /* PFE.dp   -> PFE.dict.here */
+    p4_byte_t* base;     /* PFE.dict -> PFE.dict.base */
+    p4_byte_t* limit;    /* PFE.dictlimit -> PFE.dict.limit */
 };
 
 
@@ -224,7 +240,7 @@ struct p4_Dictionary
 struct p4_Session
 {				
     int argc;
-    char** argv;
+    char const ** argv;
     unsigned    isnotatty:2,	/* running in canonical mode */
 	        stdio:1,	/* standard input isn't-tty: work as filter */
 	        caps_on:1,	/* exchange lower and upper case chars */
@@ -240,39 +256,32 @@ struct p4_Session
                 upper_case_on:1,/* make lower case words find upper case */
                 lastbit:1;      /* last bit */
     int	cols, rows;	/* size of screen */
-    p4ucell	total_size;
-    p4ucell	stack_size;
-    p4ucell	ret_stack_size;
-    p4ucell	max_files;
-    p4ucell	pockets;
-    char const*	include_file;	/* file to include after boot */
-    char const*	incpaths;
-    char const*	incext;
-    char const*	blkpaths;
-    char const*	blkext;
-    char const*	editor;		/* preferred ASCII text file editor */
+    p4ucelll	total_size;
+    p4ucelll	stack_size;
+    p4ucelll	ret_stack_size;
+    p4ucelll	unused_max_files; /* use option_value "#files" */
+    p4ucelll	unused_pockets;   /* use option_value "#pockets" */
+    /* quick access to option-ext variables... (if any) */
+    char**      unused_boot_include; /* use option_string "SCRIPT-FILE" */
+    /* we keep the following entries only to speed up runtime processing: */
+    char const** inc_paths;     /* usually points to "INC-PATH" */
+    char const** inc_ext;       /* usually points to "INC-EXT" */
+    char const** blk_paths;     /* usually points to "BLK-PATH" */
+    char const** blk_ext;       /* usually points to "BLK-EXT" */
+    char const** lib_paths;     /* usually points to "LIB-PATH" */
+    /*                 _editor; // use option_string("$EDITOR") */
         
     /* for VMs */
     int     cpus;          /* how many cpus do we have in this tread */
 
-    char const* prefix;
-    char const* bootname;
-    char *      bootcommand;
-    char **	optv;
-    struct { 
-	unsigned include_file	: 1;
-	unsigned incpaths	: 1;
-	unsigned incext		: 1;
-	unsigned blkpaths	: 1;
-	unsigned blkext		: 1;
-	unsigned editor		: 1;
-	unsigned prefix         : 1;
-	unsigned bootcommand    : 1;
-	unsigned optv		: 1;
-    } heap;
+    char const** unused_prefix;      /* use option_string("PREFIX-DIR") */
+    char const** boot_name;          /* points to argv[0] usually... */
+    char const** unused_bootcommand; /* use option_string("BOOT-INIT") */
+    char const** optv;        
+    p4cell       unused_heap;        /* obsoleted bitfield */
 
-    p4ucell     history_size;
-    p4ucell     wordlists;
+    p4ucell     optc;
+    unsigned    wordlists;       /* p4ucell might be 64bit (16bit is okay) */
     void*       modules;         /* p4Words* : dl-internal / dl-ext */
     p4ucell     padding[4];      /* padding cells for binary compatibility */
     
@@ -282,23 +291,14 @@ struct p4_Session
     /* newstyle options support via option-ext */
     struct
     {
-        p4char* last;
-        p4char* dp;
-        p4char* dict;
-        p4char* dictlimit;
-        p4char* link;
-        p4char  space[280]; /* atleast one string */
+        p4_namebuf_t* last;
+        p4_namebuf_t* dp;
+        p4_byte_t* dict;
+        p4_byte_t* dictlimit;
+        p4_byte_t  space[284]; /* atleast a few headers... */
     } opt; /* must be last in this structure !! */
 };
 
-/* actually, the following two defines are the same when p4TH == &p4_reg */
-/* but for debugging, the .c -> .i precompiler output is better readable */
-#ifdef PFE_WITH_STATIC_REGS
-#define PFE p4_reg
-#else  
-#define PFE (*p4TH)
-#endif
-
 #ifndef P4_MOPTRS
 #define P4_MOPTRS 128
 #endif
@@ -316,19 +316,19 @@ struct p4_Thread
     /* p4_cleanup() will automatically free this field, so the base
        memory *must* be the last pointer in the field.
     */
-    p4char *dp;			/* actual top of the dictionary */
+    p4_byte_t *dp;		/* actual top of the dictionary */
 
-    p4char* dict;		/*  dictionary */
-    p4char* dictlimit;
+    p4_byte_t* dict;		/*  dictionary */
+    p4_byte_t* dictlimit;
     p4cell* stack;		/*  data stack */
     p4cell* s0;
     double *fstack;		/*  floating point stack */
     double* f0;
     p4xcode** rstack;		/*  return stack */
     p4xcode**  r0;
-    char (*pockets)[P4_POCKET_SIZE];	/*  POCKET for interactive S" */
-    char *tib;			/*  TIB */
-    char* tib_end;
+    p4_pocket_t* pockets_ptr;	/*  POCKET for interactive S" */
+    p4_char_t *tib;		/*  TIB terminal input buffer */
+    p4_char_t* tib_end;
     char* history;		/*  command line history buffer */
     char* history_top;
     p4_File* files;		/*  files */
@@ -343,7 +343,7 @@ struct p4_Thread
     double*   fp; /* the floating point stack pointer */
 
 /* jmp_buf */
-    jmp_buf loop;          /* QUIT and ABORT do a THROW which longjmp() */
+    p4_jmp_buf loop;       /* QUIT and ABORT do a THROW which longjmp() */
        			   /* here thus C-stack gets cleaned up too */
 /*Options*/
     int nr;                 /* this cpu's id (mostly an optional feature)*/
@@ -352,8 +352,8 @@ struct p4_Thread
 #define PFE_set (*PFE.set)
     
 /*Dict*/
-    p4char *fence;		/* can't forget below that address */
-    char *last;			/* NFA of most recently CREATEd header */
+    p4_byte_t *fence;		/* can't forget below that address */
+    p4_namebuf_t *last;		/* NFA of most recently CREATEd header */
 
     p4_Wordl *voc_link;		/* link to chained word lists */
     p4_Wordl **context;	        /* dictionary search order */
@@ -361,19 +361,19 @@ struct p4_Thread
     p4_Wordl *current;		/* points to vocabulary in extension */
     p4_Wordl **dforder;	        /* default dictionary search order */
     p4_Wordl *dfcurrent;        /* default definition wordlist */
-    char *hld;			/* auxiliary pointer for number output */
+    p4_char_t *hld;		/* auxiliary pointer for number output */
     p4cell dpl;			/* position of input decimal point */
 
     p4_Input input;		/* active input source, full specification */
     p4_Input input_err;		/* input specification when error occurred */
     p4_Iframe *saved_input;	/* links to chain of saved input specs */
-    p4_Except *cAtch;		/* links to chain of CATCHed words */
+    p4_Except *catchframe;	/* links to chain of CATCHed words */
     p4ucell span;		/* char count of last EXPECT */
     p4ucell scr;		/* latest LISTed block number */
     p4cell out;			/* current output column on screen */
     p4cell state;		/* interpreting (0) or compiling (-1) */
     p4cell *locals;		/* number of locals in current def. */
-    char (*local)[P4_LOCALS];	/* names of locals in current def. */
+    char (*local)[P4_LOCALS];   /* names of locals in current def. */
     p4cell *csp;		/* compiler security, saves sp here */
     p4ucell base;		/* of number i/o conversion */
     p4cell precision;		/* floating point output precision */
@@ -396,7 +396,7 @@ struct p4_Thread
     p4cell redefined_msg;	/* no `"xxx" is redefined' msg if false */
     p4cell float_input;		/* don't try floating pt input when false */
     p4cell reset_order;		/* if true: reset search order on ABORT */
-    p4cell pocket;		/* which pocket to use next */
+    p4_pocket_t* pocket;	/* which pocket to use next */
 
     p4_File *stdIn;		/* C-library standard files */
     p4_File *stdOut;		/* mapped to Forth-files */
@@ -414,17 +414,17 @@ struct p4_Thread
 /* main-sub / dict-sub */
     int exitcode;
     void (*system_terminal)(void);
-    void (*atexit_cleanup)(void);          /* free this thread */
-    int atexit_running;         /* p4_atexit_cleanup called... */
-    p4_Wordl *atexit_wl;	/* atexit dictionary holder */
-    char* volatile forget_dp;   /* temporary of forget */
+    void (*atexit_cleanup)(void);    /* free this thread */
+    int atexit_running;              /* p4_atexit_cleanup called... */
+    p4_Wordl *atexit_wl;	     /* atexit dictionary holder */
+    p4_byte_t* volatile forget_dp;   /* temporary of forget */
 
 /* term.h */ 
     int rows, cols;		/* size of text screen */
     int xmax, ymax;             /* size of graphics window in pixels */
     
-    p4char keybuf [8];
-    p4char* keyptr;
+    p4_char_t keybuf [8];
+    p4_char_t* keyptr;
     
 /* term*.c */
     void* priv;         	/* private term area, better be also in p[] */
@@ -485,19 +485,19 @@ struct p4_Thread
     p4_decompile_func_t decompile[3];
 
 /* go at quoting... */
-    p4char quoted_parse;
-    p4char unused_flag1;
-    p4char unused_flag2;
-    p4char unused_flag3;
+    p4_char_t quoted_parse;
+    p4_char_t unused_flag1;
+    p4_char_t unused_flag2;
+    p4_char_t unused_flag3;
 
     struct {
-	p4char* ptr;
-	p4ucell len;
-    } word;
+	const p4_char_t* ptr;
+	unsigned len;          /* p4ucell is 8byte on x86_64 but */
+    } word;                    /* parsing is not exceeding 16bit anyway */
 
     p4xt application;		/* word to run initially or 0 */
 
-    p4char* last_here;         /* set in interpret and used in abort */
+    p4_byte_t* last_here;      /* set in interpret and used in abort */
 
     p4xt spy_enter;            /* see with-spy.c */
     p4xt spy_leave;            /* see with-spy.c */
@@ -506,8 +506,23 @@ struct p4_Thread
     p4_Wordl* abort_wl;        /* see engine-sub/chainlist-ext REDO-WL */
     p4_Wordl* prompt_wl;       /* see engine-sub/chainlist-ext DO-WL */
 
+    double asinh_MAX_over_4;          /* see complex-ext.c */
+    double sqrt_MAX_over_4;           /* see complex-ext.c */
+
+    p4_pocket_t* pockets_top;  /* fixme: should be moved to above... */
+
+/* stackhelp-ext.c */
+    p4_Wordl* stackhelp_wl;
+/* tools-ext.c + assembler-ext.c */
+    p4_Wordl* assembler_wl;   
+
+    p4xt    interpret_loop;       /* compiled interpret loop */
+    p4cell  interpret_compiled;   /* use it! */
+    p4cell* interpret_compile_resolve;
+    p4cell* interpret_compile_extra;
+    p4cell* interpret_compile_float;
 /* make updates safer with additional padding space, use it! */
-    p4cell padding[1];
+    p4cell padding[6];
 };
 
 # define p4_S0 PFE.s0
@@ -517,7 +532,7 @@ struct p4_Thread
 # define p4_DP          (PFE.dp)
 # define p4_HLD		(PFE.hld)
 # define p4_DPL		(PFE.dpl)
-# define p4_PAD		((char *)p4_DP + P4_MIN_HOLD)
+# define p4_PAD		((p4_char_t *)p4_DP + P4_MIN_HOLD)
 # define p4_FENCE	(PFE.fence)
 # define p4_LAST	(PFE.last)
 # define p4_VOC_LINK	(PFE.voc_link)
@@ -543,9 +558,8 @@ struct p4_Thread
 # define APPLICATION	p4_APPLICATION
 #endif
 
-# define p4_DP_CHAR     ((p4char*)(p4_DP))
+# define p4_DP_CHAR     p4_DP
 # define p4_DP_CELL     ((p4cell*)(p4_DP))
-# define p4_DP_CODE     ((p4xcode*)(p4_DP))
 
 # define p4_SOURCE_ID	(PFE.input.source_id)
 # define p4_SOURCE_FILE	((p4_File*) SOURCE_ID)
