@@ -1,18 +1,19 @@
-#ifndef _PFE_EDIT_EXT_H
-#define _PFE_EDIT_EXT_H 1034771921
-/* generated 2002-1016-1438 ../../../pfe/../mk/Make-H.pl ../../../pfe/edit-ext.c */
+#ifndef _VOL_8_SRC_CVS_PFE_33_PFE_EDIT_EXT_H
+#define _VOL_8_SRC_CVS_PFE_33_PFE_EDIT_EXT_H 1209868838
+/* generated 2008-0504-0440 /vol/8/src/cvs/pfe-33/pfe/../mk/Make-H.pl /vol/8/src/cvs/pfe-33/pfe/edit-ext.c */
 
-#include <pfe/incl-ext.h>
+#include <pfe/pfe-ext.h>
 
 /** 
  * -- simple FORTH-screenfile editor
  *
- *  Copyright (C) Tektronix, Inc. 1998 - 2000. All rights reserved.
+ *  Copyright (C) Tektronix, Inc. 1998 - 2000.
+ *  Copyright (C) 2005 - 2008 Guido U. Draheim <guidod@gmx.de>
  *
  *  @see     GNU LGPL
- *  @author  Tektronix CTE            @(#) %derived_by: guidod %
- *  @version %version: bln_mpt1!5.22 %
- *    (%date_modified: Mon Apr 08 20:17:44 2002 %)
+ *  @author  Guido U. Draheim            (modified by $Author: guidod $)
+ *  @version $Revision: 1.5 $
+ *     (modified $Date: 2008/05/04 02:57:30 $)
  *
  *  @description
  *       This is a simple fullscreen FORTH block editor.
