@@ -1,22 +1,24 @@
 #ifndef __PFE_PFE_BASE_H
-#define __PFE_PFE_BASE_H "%full_filespec: pfe-base.h~bln_mpt1!1.6:incl:bln_12xx!1 %"
-
+#define __PFE_PFE_BASE_H
 /** 
  * -- declare prototypes of the base system
  * 
- *  Copyright (C) Tektronix, Inc. 1998 - 2000. All rights reserved.
+ *  Copyright (C) Tektronix, Inc. 1998 - 2003.
+ *  Copyright (C) 2005 - 2008 Guido U. Draheim <guidod@gmx.de>
  *
  *  @see     GNU LGPL
- *  @author  Tektronix CTE             @(#) %derived_by: guidod %
- *  @version %version: bln_mpt1!1.6 %
- *    (%date_modified: Wed Apr 03 11:09:57 2002 %)
+ *  @author  Guido U. Draheim            (modified by $Author: guidod $)
+ *  @version $Revision: 1.3 $
+ *     (modified $Date: 2008/04/20 04:46:30 $)
+ *
  *  @description
  *            most of the routines of the internal forth system
  *            are declared here
  */
 /*@{*/
 
-#include <pfe/incl-ext.h>
+#include <pfe/pfe-ext.h>
+
 #include <pfe/core-sub.h>
 #include <pfe/core-ext.h>
 #include <pfe/core-mix.h>
@@ -25,12 +27,13 @@
 #include <pfe/header-sub.h>
 #include <pfe/block-sub.h>
 #include <pfe/block-ext.h>
-#include <pfe/utils-sub.h>
 #include <pfe/engine-sub.h>
+#include <pfe/engine-set.h>
 #include <pfe/exception-sub.h>
 #include <pfe/memory-sub.h>
-#include <pfe/main-sub.h>
 #include <pfe/misc-ext.h>
+#include <pfe/option-ext.h>
+#include <pfe/option-set.h>
 #include <pfe/search-order-ext.h>
 #include <pfe/signals-ext.h>
 #include <pfe/tools-sub.h>
