@@ -1,18 +1,19 @@
-#ifndef _PFE_TOOLS_SUB_H
-#define _PFE_TOOLS_SUB_H 1034771918
-/* generated 2002-1016-1438 ../../../pfe/../mk/Make-H.pl ../../../pfe/tools-sub.c */
+#ifndef _VOL_8_SRC_CVS_PFE_33_PFE_TOOLS_SUB_H
+#define _VOL_8_SRC_CVS_PFE_33_PFE_TOOLS_SUB_H 1209868837
+/* generated 2008-0504-0440 /vol/8/src/cvs/pfe-33/pfe/../mk/Make-H.pl /vol/8/src/cvs/pfe-33/pfe/tools-sub.c */
 
-#include <pfe/incl-sub.h>
+#include <pfe/pfe-sub.h>
 
 /** 
  * -- implementation words for TOOLS-EXT / TOOLS-MIX
  *
- *  Copyright (C) Tektronix, Inc. 1998 - 2001. All rights reserved.
+ *  Copyright (C) Tektronix, Inc. 1998 - 2001.
+ *  Copyright (C) 2005 - 2008 Guido U. Draheim <guidod@gmx.de>
  *
  *  @see     GNU LGPL
- *  @author  Tektronix CTE              @(#) %derived_by: guidod %
- *  @version %version:  %
- *    (%date_modified:  %)
+ *  @author  Guido U. Draheim            (modified by $Author: guidod $)
+ *  @version $Revision: 1.5 $
+ *     (modified $Date: 2008/05/04 02:57:30 $)
  *
  *  @description
  *      These are tool words used throughout the system implemenation.
